package com.game.business.mapper;

import java.util.List;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.game.business.domain.AppGameItem;
import com.game.business.dto.AppItemCountDTO;

/**
 * 游戏选项Mapper接口
 *
 * @author game
 * @date 2024-06-29
 */
public interface AppGameItemMapper extends BaseMapper<AppGameItem> {
    /**
     * 查询游戏选项
     *
     * @param id 游戏选项主键
     * @return 游戏选项
     */
    public AppGameItem selectAppGameItemById(Long id);

    /**
     * 查询游戏选项列表
     *
     * @param appGameItem 游戏选项
     * @return 游戏选项集合
     */
    public List<AppGameItem> selectAppGameItemList(AppGameItem appGameItem);

    /**
     * 新增游戏选项
     *
     * @param appGameItem 游戏选项
     * @return 结果
     */
    public int insertAppGameItem(AppGameItem appGameItem);

    /**
     * 修改游戏选项
     *
     * @param appGameItem 游戏选项
     * @return 结果
     */
    public int updateAppGameItem(AppGameItem appGameItem);

    /**
     * 修改游戏选项开奖值
     *
     * @param appGameItem 修改游戏选项开奖值
     * @return 结果
     */
    int updateLotteryCount(AppGameItem appGameItem);

    /**
     * 修改连中次数
     *
     * @param appGameItem 修改连中次数
     * @return 结果
     */
    int updateConsecutive(AppItemCountDTO appGameItem);


    /**
     * 删除游戏选项
     *
     * @param id 游戏选项主键
     * @return 结果
     */
    public int deleteAppGameItemById(Long id);

    /**
     * 批量删除游戏选项
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteAppGameItemByIds(Long[] ids);
}