package com.game.business.service;

import java.util.List;
import com.game.business.domain.AppGame;
import com.baomidou.mybatisplus.extension.service.IService;

/**
 * 游戏配置Service接口
 *
 * @author game
 * @date 2024-06-14
 */
public interface IAppGameService extends IService<AppGame> {
    /**
     * 查询游戏配置
     *
     * @param id 游戏配置主键
     * @return 游戏配置
     */
    public AppGame selectAppGameById(Long id);

    /**
     * 查询游戏配置列表
     *
     * @param appGame 游戏配置
     * @return 游戏配置集合
     */
    public List<AppGame> selectAppGameList(AppGame appGame);

    /**
     * 新增游戏配置
     *
     * @param appGame 游戏配置
     * @return 结果
     */
    public int insertAppGame(AppGame appGame);

    /**
     * 修改游戏配置
     *
     * @param appGame 游戏配置
     * @return 结果
     */
    public int updateAppGame(AppGame appGame);

    /**
     * 批量删除游戏配置
     *
     * @param ids 需要删除的游戏配置主键集合
     * @return 结果
     */
    public int deleteAppGameByIds(Long[] ids);

    /**
     * 删除游戏配置信息
     *
     * @param id 游戏配置主键
     * @return 结果
     */
    public int deleteAppGameById(Long id);

    /**
     * 根据类型查询游戏
     * @param classId
     * @return
     */
    List<AppGame> selectAppGameByClassId(Long classId);

    /**
     * 根据编码查询游戏
     * @param classId
     * @param code
     * @return
     */
    AppGame selectAppGameByClassIdAndCode(Long classId, String code);

    /**
     * 根据id查询游戏
     * @param ids
     * @return
     */
    List<AppGame> selectByIds(List<Long> ids);
}